/**
 * \file Peer2Peer.c
 *
 * \brief Peer2Peer application implementation
 *
 * Copyright (C) 2012-2013, Atmel Corporation. All rights reserved.
 *
 * \asf_license_start
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with an
 *    Atmel microcontroller product.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * \asf_license_stop
 *
 * $Id: Peer2Peer.c 8376 2013-07-25 23:57:59Z ataradov $
 *
 */

/*- Includes ---------------------------------------------------------------*/
#include "hw_config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "config.h"
#include "hal.h"
#include "phy.h"
#include "sys.h"
#include "nwk.h"
#include "sysTimer.h"
#include "halUart.h"
#include "nwkCommand.h"
#include "nwkTx.h"

/*- Definitions ------------------------------------------------------------*/
#ifdef NWK_ENABLE_SECURITY
  #define APP_BUFFER_SIZE     (NWK_MAX_PAYLOAD_SIZE - NWK_SECURITY_MIC_SIZE)
#else
  #define APP_BUFFER_SIZE     NWK_MAX_PAYLOAD_SIZE
#endif

/*- Types ------------------------------------------------------------------*/
typedef enum AppState_t
{
  APP_STATE_INITIAL,
  APP_STATE_IDLE,
} AppState_t;

/*- Prototypes -------------------------------------------------------------*/
static void appSendData(void);

/*- Variables --------------------------------------------------------------*/
static AppState_t appState = APP_STATE_INITIAL;
static SYS_Timer_t appTimer;
static NWK_DataReq_t appDataReq;
static bool appDataReqBusy = false;
static uint8_t appDataReqBuffer[APP_BUFFER_SIZE];
static uint8_t appUartBuffer[APP_BUFFER_SIZE];
static uint8_t appUartBufferPtr = 0;

AT86RF212_TestMode_E  g_at86rf212_test_mode;

/*- Implementations --------------------------------------------------------*/

/*************************************************************************//**
*****************************************************************************/
static void appDataConf(NWK_DataReq_t *req)
{
  appDataReqBusy = false;
  (void)req;
}

/*************************************************************************//**
*****************************************************************************/
static void appSendData(void)
{
    if (appDataReqBusy || 0 == appUartBufferPtr)
    return;

    memcpy(appDataReqBuffer, appUartBuffer, appUartBufferPtr);

    if(AT86RF212_TEST_MODE_TX == g_at86rf212_test_mode)
    {
        appDataReq.dstAddr = APP_ADDR_RX;
    }
    else
    {
        appDataReq.dstAddr = APP_ADDR_TX;
    }
    printf("appDataReq.dstAddr is 0x%02X\n", appDataReq.dstAddr);

    appDataReq.dstEndpoint = APP_ENDPOINT;
    appDataReq.srcEndpoint = APP_ENDPOINT;
    appDataReq.options = NWK_OPT_ENABLE_SECURITY;
    appDataReq.data = appDataReqBuffer;
    appDataReq.size = appUartBufferPtr;
    appDataReq.confirm = appDataConf;
    NWK_DataReq(&appDataReq);

    appUartBufferPtr = 0;
    appDataReqBusy = true;
}

/*************************************************************************//**
*****************************************************************************/
void HAL_UartBytesReceived(uint16_t bytes)
{
  for (uint16_t i = 0; i < bytes; i++)
  {
    uint8_t byte = HAL_UartReadByte();

    if (appUartBufferPtr == sizeof(appUartBuffer))
      appSendData();

    if (appUartBufferPtr < sizeof(appUartBuffer))
      appUartBuffer[appUartBufferPtr++] = byte;
  }

  SYS_TimerStop(&appTimer);
  SYS_TimerStart(&appTimer);
}

void HAL_ByteReceived(uint8_t byte)
{
    appUartBuffer[appUartBufferPtr++] = byte;
    appSendData();
}

/*************************************************************************//**
*****************************************************************************/
static void appTimerHandler(SYS_Timer_t *timer)
{
  appSendData();
  (void)timer;
}

/*************************************************************************//**
*****************************************************************************/
static bool appDataInd(NWK_DataInd_t *ind)
{
//  for (uint8_t i = 0; i < ind->size; i++)
//    HAL_UartWriteByte(ind->data[i]);
  return true;
}

/*************************************************************************//**
*****************************************************************************/
static void appInit(void)
{
    if(AT86RF212_TEST_MODE_TX == g_at86rf212_test_mode)
    {
        NWK_SetAddr(APP_ADDR_TX);
    }
    else
    {
        NWK_SetAddr(APP_ADDR_RX);
    }

    NWK_SetPanId(APP_PANID);
    PHY_SetChannel(APP_CHANNEL);
    PHY_SetBand(APP_BAND);
    PHY_SetModulation(APP_MODULATION);
    PHY_SetRxState(true);

    NWK_OpenEndpoint(APP_ENDPOINT, appDataInd);

    appTimer.interval = APP_FLUSH_TIMER_INTERVAL;
    appTimer.mode = SYS_TIMER_INTERVAL_MODE;
    appTimer.handler = appTimerHandler;
}

/*************************************************************************//**
*****************************************************************************/
static void APP_TaskHandler(void)
{
  switch (appState)
  {
    case APP_STATE_INITIAL:
    {
      appInit();
      appState = APP_STATE_IDLE;
    } break;

    case APP_STATE_IDLE:
      break;

    default:
      break;
  }
}

/*************************************************************************//**
*****************************************************************************/

extern NwkFrame_t *nwkTxPhyActiveFrame;

int AT86RF212_Test_Peer2Peer_main(AT86RF212_TestMode_E testMode)
{
    uint8_t ch = 0;

    DEBUG_MSG_FUNC_START;

    g_at86rf212_test_mode = testMode;

    printf("size of NwkCommandAck_t is %d\n", sizeof(NwkCommandAck_t));
    printf("size of NwkCommandRouteError_t is %d\n", sizeof(NwkCommandRouteError_t));
    printf("size of NwkCommandRouteRequest_t is %d\n", sizeof(NwkCommandRouteRequest_t));
    printf("size of NwkCommandRouteReply_t is %d\n", sizeof(NwkCommandRouteReply_t));
    printf("size of NwkFrameHeader_t is %d\n", sizeof(NwkFrameHeader_t));
    printf("size of NwkFrameMulticastHeader_t is %d\n", sizeof(NwkFrameMulticastHeader_t));

    SYS_Init();
    // HAL_UartInit(38400);

    printf("PART_NUM_REG: 0x%02X, VERSION_NUM_REG: 0x%02X\n",
        phyReadRegister(PART_NUM_REG), phyReadRegister(VERSION_NUM_REG));

    AT86RF212_GPIO_0_Interrupt_Setting(ENABLE);

    NwkFrame_t *frame;

    while (1)
    {
        if(AT86RF212_TEST_MODE_RX == g_at86rf212_test_mode)
        {
#if 1
            SYS_TaskHandler();
            // HAL_UartTaskHandler();
            APP_TaskHandler();
//#else
     //       printf("PART_NUM_REG: 0x%02X, VERSION_NUM_REG: 0x%02X\n",
     //           phyReadRegister(PART_NUM_REG), phyReadRegister(VERSION_NUM_REG));
     //       printf("TRX_STATUS_REG: 0x%02X, TRX_STATE_REG: 0x%02X\n",
     //           phyReadRegister(TRX_STATUS_REG), phyReadRegister(TRX_STATE_REG));
            Delay(10);
#endif
        }
        else
        {
#if 0
            if(FALSE == UartQueue_Serial_Is_Empty())
            {
                ch = UartQueue_Serial_DeQueue();
                printf("%c (0x%02X) is selected\n", (char)ch, ch);
                printf("PART_NUM_REG: 0x%02X, VERSION_NUM_REG: 0x%02X\n",
                    phyReadRegister(PART_NUM_REG), phyReadRegister(VERSION_NUM_REG));

                HAL_ByteReceived(ch);
            }
            
            SYS_TaskHandler();
            // HAL_UartTaskHandler();
            APP_TaskHandler();
#else
            appInit();

//            printf("PART_NUM_REG: 0x%02X, VERSION_NUM_REG: 0x%02X\n",
//                phyReadRegister(PART_NUM_REG), phyReadRegister(VERSION_NUM_REG));

            if (NULL == (frame = nwkFrameAlloc()))
            {
                printf("[ERROR] frame nwkFrameAlloc Fail !!!\n");
                return -1;
            }

            appDataReq.dstAddr = APP_ADDR_RX;
///            printf("appDataReq.dstAddr is 0x%02X\n", appDataReq.dstAddr);

            appDataReq.dstEndpoint = APP_ENDPOINT;
            appDataReq.srcEndpoint = APP_ENDPOINT;
            appDataReq.options = NWK_OPT_ENABLE_SECURITY;
            appDataReq.data = &ch;
            appDataReq.size = 1;
            appDataReq.confirm = appDataConf;

            printf("appDataReq.data is 0x%02X\n", ch);

            appDataReq.frame = frame;
            // appDataReq.state = NWK_DATA_REQ_STATE_WAIT_CONF;

            // frame->tx.confirm = nwkDataReqTxConf;
            frame->tx.control = appDataReq.options & NWK_OPT_BROADCAST_PAN_ID ? NWK_TX_CONTROL_BROADCAST_PAN_ID : 0;

            frame->header.nwkFcf.ackRequest = appDataReq.options & NWK_OPT_ACK_REQUEST ? 1 : 0;
            frame->header.nwkFcf.linkLocal = appDataReq.options & NWK_OPT_LINK_LOCAL ? 1 : 0;

            frame->header.nwkSeq = ++nwkIb.nwkSeqNum;
            frame->header.nwkSrcAddr = nwkIb.addr;
            frame->header.nwkDstAddr = appDataReq.dstAddr;
            frame->header.nwkSrcEndpoint = appDataReq.srcEndpoint;
            frame->header.nwkDstEndpoint = appDataReq.dstEndpoint;

            memcpy(frame->payload, appDataReq.data, appDataReq.size);
            frame->size += appDataReq.size;

            nwkTxFrame(frame);

            nwkTxPhyActiveFrame = frame;
            // frame->state = NWK_TX_STATE_WAIT_CONF;
            PHY_DataReq(frame->data, frame->size);

            Delay(1000);
            ch ++;
            
            nwkFrameFree(frame);
#endif
        }
    }
}

