/*
 * (C) COPYRIGHT 2012 CRZ
 *
 * File Name : slip.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 11/26/2012
 */

/* includes */

#include "hw_config.h"

/* functions */

/* SLIP special character codes */
#define END        0xC0 /* Decimal 192, Octal 0300 */ /* indicates end of packet */
#define ESC        0xDB /* Decimal 219, Octal 0333 */ /* indicates byte stuffing */
#define ESC_END    0xDC /* Decimal 220, Octal 0334 */ /* ESC ESC_END means END data byte */
#define ESC_ESC    0xDD /* Decimal 221, Octal 0335 */ /* ESC ESC_ESC means ESC data byte */

/* SEND_PACKET: sends a packet of length "len", starting at location "p". */
void SLIP_Send_Packet(unsigned char * data_p, int len)
{
    /* send an initial END character to flush out any data that may
     * have accumulated in the receiver due to line noise
     */
    UartComm_SendData_OneChar(END);

    /* for each byte in the packet, send the appropriate character
     * sequence
     */
    while(len--) {
        switch(* data_p) {
        /* if it's the same code as an END character, we send a
         * special two character code so as not to make the
         * receiver think we sent an END
         */
        case END:
            UartComm_SendData_OneChar(ESC);
            UartComm_SendData_OneChar(ESC_END);
            break;

        /* if it's the same code as an ESC character,
         * we send a special two character code so as not
         * to make the receiver think we sent an ESC
         */
        case ESC:
            UartComm_SendData_OneChar(ESC);
            UartComm_SendData_OneChar(ESC_ESC);
            break;

        /* otherwise, we just send the character */
        default:
            UartComm_SendData_OneChar(* data_p);
        }

        data_p ++;
    }

    /* tell the receiver that we're done sending the packet */
    UartComm_SendData_OneChar(END);
}

/* RECV_PACKET: receives a packet into the buffer located at "p".
 *   If more than len bytes are received, the packet will be truncated.
 *   Returns the number of bytes stored in the buffer.
 */
int SLIP_Receive_Packet(unsigned char * data_p, int len)
{
    int8_t  ret = 0;
    uint8_t ch;
    int received = 0;

    /* sit in a loop reading bytes until we put together a whole packet.
    * Make sure not to copy them into the packet if we run out of room.
    */
    while(1)
    {
        /* get a character to process */
        ret = UartComm_GetChar_Timeout(&ch);
        if(ret < 0)
            return -1;

        /* handle bytestuffing if necessary
        */
        switch(ch)
        {
        /* if it's an END character then we're done with the packet */
        case END:
            /* a minor optimization: if there is no data in the packet, ignore it.
             * This is meant to avoid bothering IP with all the empty packets
             * generated by the duplicate END characters
             * which are in turn sent to try to detect line noise.
             */
            if(received)
                return received;
            else
                break;

        /* if it's the same code as an ESC character, wait and get another character
         * and then figure out what to store in the packet based on that.
         */
        case ESC:
            /* get a character to process */
            ret = UartComm_GetChar_Timeout(&ch);
            if(ret < 0)
                return -1;

            /* if "ch" is not one of these two, then we have a protocol violation.
             * The best bet seems to be to leave the byte alone and
             * just stuff it into the packet
             */
            switch(ch)
            {
            case ESC_END:
                ch = END;
                break;
            case ESC_ESC:
                ch = ESC;
                break;
            default:
                DEBUG_MSG_FUNC_ERROR;
                return -1;
            }

        /* here we fall into the default handler and let it store the character for us */
        default:
            if(received < len)
                data_p[received++] = ch;
        }
    }
}

